import httplib

import requests


class TeraVMClient(object):
    def __init__(self, address, user=None, password=None, scheme="https", port=443, verify_ssl=False):
        """
        :param str address: controller IP address
        :param str user: controller username
        :param str password: controller password
        :param str scheme: protocol (http|https)
        :param int port: controller port
        :param bool verify_ssl: whether SSL cert will be verified or not
        """
        self._base_url = "{}://{}:{}".format(scheme, address, port)
        # self._auth = HTTPBasicAuth(username=user, password=password)
        self._auth = None
        self._headers = {"Accept": "application/vnd.cobham.v1+json"}
        self._verify_ssl = verify_ssl

    def _do_get(self, path, raise_for_status=True, **kwargs):
        """Basic GET request client method

        :param str path: path for the request
        :param dict kwargs: additional kwarg that would be passed to the requests lib
        :rtype: requests.Response
        """
        url = "{}/{}".format(self._base_url, path)
        kwargs.update({"verify": self._verify_ssl})
        resp = requests.get(url=url, auth=self._auth, headers=self._headers, **kwargs)
        raise_for_status and resp.raise_for_status()
        return resp

    def _do_post(self, path, raise_for_status=True, **kwargs):
        """Basic POST request client method

        :param str path: path for the request
        :param dict kwargs: additional kwarg that would be passed to the requests lib
        :rtype: requests.Response
        """
        url = "{}/{}".format(self._base_url, path)
        kwargs.update({"verify": self._verify_ssl})
        resp = requests.post(url=url, auth=self._auth, headers=self._headers, **kwargs)
        raise_for_status and resp.raise_for_status()
        return resp

    def _do_put(self, path, raise_for_status=True, **kwargs):
        """Basic PUT request client method

        :param str path: path for the request
        :param dict kwargs: additional kwarg that would be passed to the requests lib
        :rtype: requests.Response
        """
        url = "{}/{}".format(self._base_url, path)
        kwargs.update({"verify": self._verify_ssl})
        resp = requests.put(url=url, auth=self._auth, headers=self._headers, **kwargs)
        raise_for_status and resp.raise_for_status()
        return resp

    def _do_delete(self, path, raise_for_status=True, **kwargs):
        """Basic DELETE request client method

        :param str path: path for the request
        :param dict kwargs: additional kwarg that would be passed to the requests lib
        :rtype: requests.Response
        """
        url = "{}/{}".format(self._base_url, path)
        kwargs.update({"verify": self._verify_ssl})
        resp = requests.delete(url=url, auth=self._auth, headers=self._headers, **kwargs)
        raise_for_status and resp.raise_for_status()
        return resp

    def check_if_service_is_deployed(self, logger):
        """

        :return:
        """
        try:
            resp = self._do_get(path="v1/legacy-ui/ui/index.html", raise_for_status=False)
        except requests.exceptions.ConnectionError:
            return False

        if resp.status_code == httplib.OK and "executive management ip" in resp.content.lower():
            return True

    def configure_executive_server(self, ip_addr):
        """"""
        data = {
            "executiveMachineIP": ip_addr
        }

        resp = self._do_post(path="v1/legacy-ui/application/settings/executive-ip", data=data)
        return resp

    def get_modules_info(self):
        """Get test modules and ports information

        :return:
        """
        response = self._do_get(path="v1/poolmanager/testModules")
        data = response.json()

        return data["testModules"]



from cloudshell.cli.cli_service_impl import CliServiceImpl
from cloudshell.cli.command_mode_helper import CommandModeHelper
from cloudshell.cli.command_template.command_template_executor import CommandTemplateExecutor
from cloudshell.devices.cli_handler_impl import CliHandlerImpl

from cloudshell.traffic.teravm.cli import ctrl_command_templates
from cloudshell.traffic.teravm.cli.ctrl_command_modes import CliCommandMode
from cloudshell.traffic.teravm.cli.ctrl_command_modes import DefaultCommandMode


class TeraVMControllerCliHandler(CliHandlerImpl):
    def __init__(self, cli, resource_config, logger, api, open_automation_auth=True):
        """

        :param cloudshell.cli.cli.CLI cli: CLI object
        :param traffic.teravm.controller.configuration_attributes_structure.TrafficGeneratorControllerResource resource_config:
        :param logging.Logger logger:
        :param cloudshell.api.cloudshell_api.CloudShellAPISession api: cloudshell API object
        :param bool enable_open_automation_auth:
        """
        super(TeraVMControllerCliHandler, self).__init__(cli, resource_config, logger, api)
        self._modes = CommandModeHelper.create_command_mode()
        self._open_automation_auth = open_automation_auth

    def on_session_start(self, session, logger):
        """

        :param session:
        :param logger:
        :return:
        """
        # open automation authorization if needed
        if self._open_automation_auth:
            self._logger.info("Open Automation Authorization")
            test_password = self._api.DecryptPassword(self.resource_config.test_user_password).Value

            if test_password:
                cli_service = CliServiceImpl(session=session, command_mode=self.default_mode, logger=logger)
                command = CommandTemplateExecutor(cli_service=cli_service,
                                                  command_template=ctrl_command_templates.OPEN_AUTOMATION_AUTHORIZATION,
                                                  action_map={
                                                      "[Pp]assword for user":
                                                          lambda session, logger: session.send_line(
                                                              test_password, logger)
                                                  })

                command.execute_command(test_user=self.resource_config.test_user)

    @property
    def default_mode(self):
        return self._modes[DefaultCommandMode]

    @property
    def cli_mode(self):
        return self._modes[CliCommandMode]

    @property
    def enable_mode(self):
        return self.default_mode

    @property
    def config_mode(self):
        return self.default_mode



from collections import OrderedDict

from cloudshell.cli.command_template.command_template import CommandTemplate

from cloudshell.traffic.teravm.exceptions import TestGroupDoesNotExist


LICENSE_SERVER_PORT = 5053
LICENSE_SERVER_WEB_INTERFACE_PORT = 5054
DEFAULT_ERROR_MAP = OrderedDict((("ERROR:", "Error happens while executing CLI command"),
                                 ("Could not check out the required", "Failed to acquire teravm license"),
                                 ("[F]atal [E]rror", "Error happens while executing CLI command")))


def raise_test_group_exception(session, logger):
    """

    :param cloudshell.cli.session.session.Session:
    :param logging.Logger logger:
    :return:
    """
    raise TestGroupDoesNotExist()


def prepare_error_map(error_map=None):
    """

    :param collections.OrderedDict error_map:
    :rtype: collections.OrderedDict
    """
    if error_map is None:
        error_map = OrderedDict()

    error_map.update(DEFAULT_ERROR_MAP)
    return error_map


DELETE_TEST_GROUP = CommandTemplate("deleteTestGroup {test_group_name} -u {user}",
                                    action_map=OrderedDict(((r"[Tt]est [Gg]roup.*does not exist",
                                                             raise_test_group_exception),)),
                                    error_map=prepare_error_map())

IMPORT_TEST_GROUP = CommandTemplate("importTestGroup // {test_group_file} -u {user}",
                                    error_map=prepare_error_map())

START_TEST_GROUP = CommandTemplate("startTestGroup {test_group_name} -u {user}",
                                   error_map=prepare_error_map(OrderedDict(
                                       ((r"[Tt]est [Gg]roup.*does not exist", "Test Group does not exist. Be sure "
                                                                              "that you have loaded configuration"),
                                        (r"[Ss]ystem is already running", "Test Group was already started")))))

STOP_TEST_GROUP = CommandTemplate("stopTestGroup {test_group_name} -u {user}",
                                  error_map=prepare_error_map(OrderedDict(
                                      ((r"Cannot stop [Tt]est [Gg]roup as it is not currently running",
                                        "Cannot stop Test Group as it is not currently running"),
                                       (r"[Tt]est [Gg]roup.*does not exist", "Test Group does not exist. Be sure that "
                                                                             "you have loaded configuration")))))


SAVE_RESULTS = CommandTemplate("saveTestGroupHistoricalDetailedResults {test_group_name} {destination_file} -u {user}",
                               error_map=prepare_error_map(OrderedDict(
                                   ((r"[Tt]est [Gg]roup.*does not exist", "Test Group does not exist. Be sure that "
                                                                          "you have loaded configuration"),
                                    (r"There is no current or last run", "There are no stats available for Export. "
                                                                         "Be sure that you have started tests")))))

CONFIGURE_LICENSE_SERVER = CommandTemplate("configureTvmLicensing LicenseServer {license_server_ip} %s %s"
                                           % (LICENSE_SERVER_PORT, LICENSE_SERVER_WEB_INTERFACE_PORT),
                                           error_map=prepare_error_map())

OPEN_AUTOMATION_AUTHORIZATION = CommandTemplate("cli -u {test_user} openAutomationAuthorizationSession",
                                                error_map=prepare_error_map())

REMOVE_FILE = CommandTemplate("rm {file_name}")



from collections import OrderedDict

from cloudshell.cli.command_mode import CommandMode


class DefaultCommandMode(CommandMode):
    PROMPT = r':~\$'
    ENTER_COMMAND = ''
    EXIT_COMMAND = ''

    def __init__(self):
        super(DefaultCommandMode, self).__init__(DefaultCommandMode.PROMPT,
                                                 DefaultCommandMode.ENTER_COMMAND,
                                                 DefaultCommandMode.EXIT_COMMAND)


class CliCommandMode(CommandMode):
    PROMPT = r'cli>'
    ENTER_COMMAND = 'cli'
    EXIT_COMMAND = '\x03'  # Ctrl-C code

    def __init__(self):
        super(CliCommandMode, self).__init__(
            CliCommandMode.PROMPT,
            CliCommandMode.ENTER_COMMAND,
            CliCommandMode.EXIT_COMMAND,
            enter_error_map=self.enter_error_map())

    def enter_error_map(self):
        return OrderedDict((('command not found', 'TeraVM CLI is not installed'),))


CommandMode.RELATIONS_DICT = {
    DefaultCommandMode: {
        CliCommandMode: {}
    }
}
